@@ -19,12 +19,13 @@ Wrappers for the new NCBI BLAST+ tools (written in C++):
  - NcbirpstblastnCommandline - Translated Reverse Position Specific BLAST
  - NcbideltablastCommandline - Protein-Protein domain enhanced lookup time accelerated blast
  - NcbiblastformatterCommandline - Convert ASN.1 to other BLAST output formats
+ - NcbimakeblastdbCommandline - Application to create BLAST databases
 
 For further details, see:
 
 Camacho et al. BLAST+: architecture and applications
 BMC Bioinformatics 2009, 10:421
-doi:10.1186/1471-2105-10-421
+https://doi.org/10.1186/1471-2105-10-421
 """
 from __future__ import print_function
 
@@ -99,7 +100,10 @@ class _NcbibaseblastCommandline(AbstractCommandline):
         AbstractCommandline.__init__(self, cmd, **kwargs)
 
     def _validate_incompatibilities(self, incompatibles):
-        """Used by the BLAST+ _validate method (PRIVATE)."""
+        """Validate parameters for incompatibilities (PRIVATE).
+
+        Used by the _validate method.
+        """
         for a in incompatibles:
             if self._get_parameter(a):
                 for b in incompatibles[a]:
@@ -152,7 +156,8 @@ class _NcbiblastCommandline(_NcbibaseblastCommandline):
             _Option(["-gilist", "gilist"],
                     """Restrict search of database to list of GI's.
 
-                    Incompatible with: negative_gilist, seqidlist, remote, subject, subject_loc""",
+                    Incompatible with: negative_gilist, seqidlist, negative_seqidlist,
+                    remote, subject, subject_loc""",
                     filename=True,
                     equate=False),
             _Option(["-negative_gilist", "negative_gilist"],
@@ -167,6 +172,12 @@ class _NcbiblastCommandline(_NcbibaseblastCommandline):
                     Incompatible with: gilist, negative_gilist, remote, subject, subject_loc""",
                     filename=True,
                     equate=False),
+            _Option(["-negative_seqidlist", "negative_seqidlist"],
+                    """Restrict search of database to everything except the listed SeqID's.
+
+                    Incompatible with: gilist, seqidlist, remote, subject, subject_loc""",
+                    filename=True,
+                    equate=False),
             _Option(["-entrez_query", "entrez_query"],
                     "Restrict search with the given Entrez query (requires remote).",
                     equate=False),
@@ -281,7 +292,9 @@ class _Ncbiblast2SeqCommandline(_NcbiblastCommandline):
             _Option(["-subject", "subject"],
                     """Subject sequence(s) to search.
 
-                    Incompatible with: db, gilist, negative_gilist.
+                    Incompatible with: db, gilist, seqidlist, negative_gilist,
+                    negative_seqidlist, db_soft_mask, db_hard_mask
+
                     See also subject_loc.""",
                     filename=True,
                     equate=False),
@@ -289,7 +302,7 @@ class _Ncbiblast2SeqCommandline(_NcbiblastCommandline):
                     """Location on the subject sequence (Format: start-stop).
 
                     Incompatible with: db, gilist, seqidlist, negative_gilist,
-                    db_soft_mask, db_hard_mask, remote.
+                    negative_seqidlist, db_soft_mask, db_hard_mask, remote.
 
                     See also subject.""",
                     equate=False),
@@ -398,6 +411,7 @@ class NcbiblastpCommandline(_NcbiblastMain2SeqCommandline):
     """
 
     def __init__(self, cmd="blastp", **kwargs):
+        """Initialize the class."""
         self.parameters = [
             # General search options:
             _Option(["-task", "task"],
@@ -462,6 +476,7 @@ class NcbiblastnCommandline(_NcbiblastMain2SeqCommandline):
     """
 
     def __init__(self, cmd="blastn", **kwargs):
+        """Initialize the class."""
         self.parameters = [
             # Input query options:
             _Option(["-strand", "strand"],
@@ -579,6 +594,7 @@ class NcbiblastxCommandline(_NcbiblastMain2SeqCommandline):
     """
 
     def __init__(self, cmd="blastx", **kwargs):
+        """Initialize the class."""
         self.parameters = [
             # Input query options:
             _Option(["-task", "task"],
@@ -665,6 +681,7 @@ class NcbitblastnCommandline(_NcbiblastMain2SeqCommandline):
     """
 
     def __init__(self, cmd="tblastn", **kwargs):
+        """Initialize the class."""
         self.parameters = [
             # General search options:
             _Option(["-task", "task"],
@@ -749,6 +766,7 @@ class NcbitblastxCommandline(_NcbiblastMain2SeqCommandline):
     """
 
     def __init__(self, cmd="tblastx", **kwargs):
+        """Initialize the class."""
         self.parameters = [
             # Input query options:
             _Option(["-strand", "strand"],
@@ -808,6 +826,7 @@ class NcbipsiblastCommandline(_Ncbiblast2SeqCommandline):
     """
 
     def __init__(self, cmd="psiblast", **kwargs):
+        """Initialize the class."""
         self.parameters = [
             # General search options:
             _Option(["-matrix", "matrix"],
@@ -937,6 +956,7 @@ class NcbirpsblastCommandline(_NcbiblastCommandline):
     """
 
     def __init__(self, cmd="rpsblast", **kwargs):
+        """Initialize the class."""
         # TODO - remove the -word_size argument as per BLAST+ 2.2.30
         # (BLAST team say it should never have been included, since
         # the word size is set when building the domain database.)
@@ -1017,6 +1037,7 @@ class NcbirpstblastnCommandline(_NcbiblastCommandline):
     """
 
     def __init__(self, cmd="rpstblastn", **kwargs):
+        """Initialize the class."""
         # TODO - remove the -word_size argument as per BLAST+ 2.2.30
         # (BLAST team say it should never have been included, since
         # the word size is set when building the domain database.)
@@ -1094,6 +1115,7 @@ class NcbiblastformatterCommandline(_NcbibaseblastCommandline):
     """
 
     def __init__(self, cmd="blast_formatter", **kwargs):
+        """Initialize the class."""
         self.parameters = [
             # Input options
             _Option(["-rid", "rid"],
@@ -1136,6 +1158,7 @@ class NcbideltablastCommandline(_Ncbiblast2SeqCommandline):
     """
 
     def __init__(self, cmd="deltablast", **kwargs):
+        """Initialize the class."""
         self.parameters = [
             # General search options:
             _Option(["-matrix", "matrix"],
@@ -1212,8 +1235,146 @@ class NcbideltablastCommandline(_Ncbiblast2SeqCommandline):
         _Ncbiblast2SeqCommandline.__init__(self, cmd, **kwargs)
 
 
+class NcbimakeblastdbCommandline(AbstractCommandline):
+    """Wrapper for the NCBI BLAST+ program makeblastdb.
+
+    This is a wrapper for the NCBI BLAST+ makeblastdb application
+    to create BLAST databases. By default, this creates a blast database
+    with the same name as the input file. The default output location
+    is the same directory as the input.
+
+    >>> from Bio.Blast.Applications import NcbimakeblastdbCommandline
+    >>> cline = NcbimakeblastdbCommandline(dbtype="prot",
+    ...                                    input_file="NC_005816.faa")
+    >>> cline
+    NcbimakeblastdbCommandline(cmd='makeblastdb', dbtype='prot', input_file='NC_005816.faa')
+    >>> print(cline)
+    makeblastdb -dbtype prot -in NC_005816.faa
+
+    You would typically run the command line with cline() or via the Python
+    subprocess module, as described in the Biopython tutorial.
+    """
+
+    def __init__(self, cmd="makeblastdb", **kwargs):
+        """Initialize the class."""
+        self.parameters = [
+            # Basic input options
+            _Switch(["-h", "h"],
+                    "Print USAGE and DESCRIPTION; ignore other arguments."),
+            _Switch(["-help", "help"],
+                    "Print USAGE, DESCRIPTION and ARGUMENTS description; "
+                    "ignore other arguments."),
+            _Switch(["-version", "version"],
+                    "Print version number;  ignore other arguments."),
+            # Output configuration options
+            _Option(["-out", "out"],
+                    "Output file for alignment.",
+                    filename=True,
+                    equate=False),
+            # makeblastdb specific options
+            _Option(["-dbtype", "dbtype"],
+                    "Molecule type of target db ('nucl' or 'prot')",
+                    equate=False,
+                    is_required=True,
+                    checker_function=lambda x: x == 'nucl' or x == 'prot'),
+            _Option(["-in", "input_file"],
+                    "Input file/database name",
+                    filename=True,
+                    equate=False),
+            _Option(["-input_type", "input_type"],
+                    "Type of the data specified in input_file. "
+                    "Default = 'fasta'. Added in BLAST 2.2.26.",
+                    filename=False,
+                    equate=False,
+                    checker_function=self._input_type_checker),
+            _Option(["-title", "title"],
+                    "Title for BLAST database",
+                    filename=False,
+                    equate=False),
+            _Switch(["-parse_seqids", "parse_seqids"],
+                    "Option to parse seqid for FASTA input if set, for all "
+                    "other input types seqids are parsed automatically"),
+            _Switch(["-hash_index", "hash_index"],
+                    "Create index of sequence hash values."),
+            _Option(["-mask_data", "mask_data"],
+                    "Comma-separated list of input files containing masking "
+                    "data as produced by NCBI masking applications "
+                    "(e.g. dustmasker, segmasker, windowmasker)",
+                    filename=True,
+                    equate=False),
+            _Option(["-mask_id", "mask_id"],
+                    "Comma-separated list of strings to uniquely identify the "
+                    "masking algorithm",
+                    filename=False,
+                    equate=False),
+            _Option(["-mask_desc", "mask_desc"],
+                    "Comma-separated list of free form strings to describe "
+                    "the masking algorithm details",
+                    filename=False,
+                    equate=False),
+            _Switch(["-gi_mask", "gi_mask"],
+                    "Create GI indexed masking data."),
+            _Option(["-gi_mask_name", "gi_mask_name"],
+                    "Comma-separated list of masking data output files.",
+                    filename=False,
+                    equate=False),
+            _Option(["-max_file_sz", "max_file_sz"],
+                    "Maximum file size for BLAST database files. "
+                    "Default = '1GB'",
+                    filename=False,
+                    equate=False),
+            _Option(["-logfile", "logfile"],
+                    "File to which the program log should be redirected",
+                    filename=True,
+                    equate=False),
+            _Option(["-taxid", "taxid"],
+                    "Taxonomy ID to assign to all sequences",
+                    filename=False,
+                    equate=False,
+                    checker_function=lambda x: type(x)(int(x)) == x),
+            _Option(["-taxid_map", "taxid_map"],
+                    "Text file mapping sequence IDs to taxonomy IDs. "
+                    "Format:<SequenceId> <TaxonomyId><newline>",
+                    filename=True,
+                    equate=False),
+        ]
+        AbstractCommandline.__init__(self, cmd, **kwargs)
+
+    def _input_type_checker(command, x):
+        return x in ('asn1_bin', 'asn1_txt', 'blastdb', 'fasta')
+
+    def _validate(self):
+        incompatibles = {"mask_id": ["gi_mask"],
+                         "gi_mask": ["mask_id"],
+                         "taxid": ["taxid_map"]}
+
+        # Copied from _NcbibaseblastCommandline class above.
+        # Code repeated here for python2 and 3 comptaibility,
+        # because this is not a _NcbibaseblastCommandline subclass.
+        for a in incompatibles:
+            if self._get_parameter(a):
+                for b in incompatibles[a]:
+                    if self._get_parameter(b):
+                        raise ValueError("Options %s and %s are incompatible."
+                                         % (a, b))
+
+        if self.mask_id and not self.mask_data:
+            raise ValueError("Option mask_id requires mask_data to be set.")
+        if self.mask_desc and not self.mask_id:
+            raise ValueError("Option mask_desc requires mask_id to be set.")
+        if self.gi_mask and not self.parse_seqids:
+            raise ValueError("Option gi_mask requires parse_seqids to be set.")
+        if self.gi_mask_name and not (self.mask_data and self.gi_mask):
+            raise ValueError("Option gi_mask_name requires mask_data and "
+                             "gi_mask to be set.")
+        if self.taxid_map and not self.parse_seqids:
+            raise ValueError("Option taxid_map requires parse_seqids "
+                             "to be set.")
+        AbstractCommandline._validate(self)
+
+
 def _test():
-    """Run the Bio.Blast.Applications module's doctests."""
+    """Run the Bio.Blast.Applications module's doctests (PRIVATE)."""
     import doctest
     doctest.testmod(verbose=1)
 
