@@ -1,5 +1,5 @@
 # Copyright 2000 by Jeffrey Chang, Brad Chapman.  All rights reserved.
-# Copyright 2006-2016 by Peter Cock.  All rights reserved.
+# Copyright 2006-2017 by Peter Cock.  All rights reserved.
 #
 # This code is part of the Biopython distribution and governed by its
 # license.  Please see the LICENSE file that should have been included
@@ -27,18 +27,16 @@ The following internal classes are not intended for direct use and may
 be deprecated in a future release.
 
 Classes:
-
-    - Iterator              Iterate through a file of GenBank entries
-    - ErrorFeatureParser    Catch errors caused during parsing.
-    - FeatureParser         Parse GenBank data in SeqRecord and SeqFeature objects.
-    - RecordParser          Parse GenBank data into a Record object.
+ - Iterator              Iterate through a file of GenBank entries
+ - ErrorFeatureParser    Catch errors caused during parsing.
+ - FeatureParser         Parse GenBank data in SeqRecord and SeqFeature objects.
+ - RecordParser          Parse GenBank data into a Record object.
 
 Exceptions:
-
-    - ParserFailureError    Exception indicating a failure in the parser (ie.
-      scanner or consumer)
-    - LocationParserError   Exception indicating a problem with the spark based
-      location parser.
+ - ParserFailureError    Exception indicating a failure in the parser (ie.
+   scanner or consumer)
+ - LocationParserError   Exception indicating a problem with the spark based
+   location parser.
 
 """
 from __future__ import print_function
@@ -147,7 +145,7 @@ assert not _re_simple_compound.match("join(complement(149815..150200),complement
 assert not _re_complex_location.match("join(complement(149815..150200),complement(293787..295573),NC_016402.1:6618..6676,181647..181905)")
 assert not _re_simple_location.match("join(complement(149815..150200),complement(293787..295573),NC_016402.1:6618..6676,181647..181905)")
 
-_solo_bond = re.compile("bond\(%s\)" % _solo_location)
+_solo_bond = re.compile(r"bond\(%s\)" % _solo_location)
 assert _solo_bond.match("bond(196)")
 assert _solo_bond.search("bond(196)")
 assert _solo_bond.search("join(bond(284),bond(305),bond(309),bond(305))")
@@ -241,7 +239,7 @@ def _pos(pos_str, offset=0):
 
 
 def _loc(loc_str, expected_seq_length, strand):
-    """FeatureLocation from non-compound non-complement location (PRIVATE).
+    """Make FeatureLocation from non-compound non-complement location (PRIVATE).
 
     Simple examples,
 
@@ -381,14 +379,15 @@ class Iterator(object):
     Please use Bio.SeqIO.parse(..., format="gb") or Bio.GenBank.parse(...)
     for SeqRecord and GenBank specific Record objects respectively instead.
     """
+
     def __init__(self, handle, parser=None):
         """Initialize the iterator.
 
         Arguments:
+         - handle - A handle with GenBank entries to iterate through.
+         - parser - An optional parser to pass the entries through before
+           returning them. If None, then the raw entry will be returned.
 
-            - handle - A handle with GenBank entries to iterate through.
-            - parser - An optional parser to pass the entries through before
-              returning them. If None, then the raw entry will be returned.
         """
         self.handle = handle
         self._parser = parser
@@ -419,18 +418,19 @@ class Iterator(object):
             return self.__next__()
 
     def __iter__(self):
+        """Iterate over the records."""
         return iter(self.__next__, None)
 
 
 class ParserFailureError(Exception):
-    """Failure caused by some kind of problem in the parser.
-    """
+    """Failure caused by some kind of problem in the parser."""
+
     pass
 
 
 class LocationParserError(Exception):
-    """Could not Properly parse out a location from a GenBank file.
-    """
+    """Could not Properly parse out a location from a GenBank file."""
+
     pass
 
 
@@ -442,30 +442,30 @@ class FeatureParser(object):
 
     Please use Bio.SeqIO.parse(...) or Bio.SeqIO.read(...) instead.
     """
+
     def __init__(self, debug_level=0, use_fuzziness=1,
                  feature_cleaner=FeatureValueCleaner()):
         """Initialize a GenBank parser and Feature consumer.
 
         Arguments:
+         - debug_level - An optional argument that species the amount of
+           debugging information the parser should spit out. By default we have
+           no debugging info (the fastest way to do things), but if you want
+           you can set this as high as two and see exactly where a parse fails.
+         - use_fuzziness - Specify whether or not to use fuzzy representations.
+           The default is 1 (use fuzziness).
+         - feature_cleaner - A class which will be used to clean out the
+           values of features. This class must implement the function
+           clean_value. GenBank.utils has a "standard" cleaner class, which
+           is used by default.
 
-            - debug_level - An optional argument that species the amount of
-              debugging information the parser should spit out. By default we have
-              no debugging info (the fastest way to do things), but if you want
-              you can set this as high as two and see exactly where a parse fails.
-            - use_fuzziness - Specify whether or not to use fuzzy representations.
-              The default is 1 (use fuzziness).
-            - feature_cleaner - A class which will be used to clean out the
-              values of features. This class must implement the function
-              clean_value. GenBank.utils has a "standard" cleaner class, which
-              is used by default.
         """
         self._scanner = GenBankScanner(debug_level)
         self.use_fuzziness = use_fuzziness
         self._cleaner = feature_cleaner
 
     def parse(self, handle):
-        """Parse the specified handle.
-        """
+        """Parse the specified handle."""
         _consumer = _FeatureConsumer(self.use_fuzziness,
                                      self._cleaner)
         self._scanner.feed(handle, _consumer)
@@ -481,21 +481,21 @@ class RecordParser(object):
     Please use the Bio.GenBank.parse(...) or Bio.GenBank.read(...) functions
     instead.
     """
+
     def __init__(self, debug_level=0):
         """Initialize the parser.
 
         Arguments:
+         - debug_level - An optional argument that species the amount of
+           debugging information the parser should spit out. By default we have
+           no debugging info (the fastest way to do things), but if you want
+           you can set this as high as two and see exactly where a parse fails.
 
-            - debug_level - An optional argument that species the amount of
-              debugging information the parser should spit out. By default we have
-              no debugging info (the fastest way to do things), but if you want
-              you can set this as high as two and see exactly where a parse fails.
         """
         self._scanner = GenBankScanner(debug_level)
 
     def parse(self, handle):
-        """Parse the specified handle into a GenBank record.
-        """
+        """Parse the specified handle into a GenBank record."""
         _consumer = _RecordConsumer()
 
         self._scanner.feed(handle, _consumer)
@@ -508,6 +508,7 @@ class _BaseGenBankConsumer(object):
     This just helps to eliminate some duplication in things that most
     GenBank consumers want to do.
     """
+
     # Special keys in GenBank records that we should remove spaces from
     # For instance, \translation keys have values which are proteins and
     # should have spaces and newlines removed from them. This class
@@ -525,8 +526,7 @@ class _BaseGenBankConsumer(object):
 
     @staticmethod
     def _split_keywords(keyword_string):
-        """Split a string of keywords into a nice clean list.
-        """
+        """Split a string of keywords into a nice clean list (PRIVATE)."""
         # process the keywords into a python list
         if keyword_string == "" or keyword_string == ".":
             keywords = ""
@@ -540,8 +540,7 @@ class _BaseGenBankConsumer(object):
 
     @staticmethod
     def _split_accessions(accession_string):
-        """Split a string of accession numbers into a list.
-        """
+        """Split a string of accession numbers into a list (PRIVATE)."""
         # first replace all line feeds with spaces
         # Also, EMBL style accessions are split with ';'
         accession = accession_string.replace("\n", " ").replace(";", " ")
@@ -550,8 +549,7 @@ class _BaseGenBankConsumer(object):
 
     @staticmethod
     def _split_taxonomy(taxonomy_string):
-        """Split a string with taxonomy info into a list.
-        """
+        """Split a string with taxonomy info into a list (PRIVATE)."""
         if not taxonomy_string or taxonomy_string == ".":
             # Missing data, no taxonomy
             return []
@@ -573,7 +571,7 @@ class _BaseGenBankConsumer(object):
 
     @staticmethod
     def _clean_location(location_string):
-        """Clean whitespace out of a location string.
+        """Clean whitespace out of a location string (PRIVATE).
 
         The location parser isn't a fan of whitespace, so we clean it out
         before feeding it into the parser.
@@ -585,8 +583,7 @@ class _BaseGenBankConsumer(object):
 
     @staticmethod
     def _remove_newlines(text):
-        """Remove any newlines in the passed text, returning the new string.
-        """
+        """Remove any newlines in the passed text, returning the new string (PRIVATE)."""
         # get rid of newlines in the qualifier value
         newlines = ["\n", "\r"]
         for ws in newlines:
@@ -596,20 +593,18 @@ class _BaseGenBankConsumer(object):
 
     @staticmethod
     def _normalize_spaces(text):
-        """Replace multiple spaces in the passed text with single spaces.
-        """
+        """Replace multiple spaces in the passed text with single spaces (PRIVATE)."""
         # get rid of excessive spaces
         return ' '.join(x for x in text.split(" ") if x)
 
     @staticmethod
     def _remove_spaces(text):
-        """Remove all spaces from the passed text.
-        """
+        """Remove all spaces from the passed text (PRIVATE)."""
         return text.replace(" ", "")
 
     @staticmethod
     def _convert_to_python_numbers(start, end):
-        """Convert a start and end range to python notation.
+        """Convert a start and end range to python notation (PRIVATE).
 
         In GenBank, starts and ends are defined in "biological" coordinates,
         where 1 is the first base and [i, j] means to include both i and j.
@@ -631,12 +626,13 @@ class _FeatureConsumer(_BaseGenBankConsumer):
     """Create a SeqRecord object with Features to return (PRIVATE).
 
     Attributes:
-
      - use_fuzziness - specify whether or not to parse with fuzziness in
        feature locations.
      - feature_cleaner - a class that will be used to provide specialized
        cleaning-up of feature values.
+
     """
+
     def __init__(self, use_fuzziness, feature_cleaner=None):
         from Bio.SeqRecord import SeqRecord
         _BaseGenBankConsumer.__init__(self)
@@ -654,8 +650,7 @@ class _FeatureConsumer(_BaseGenBankConsumer):
         self._expected_size = None
 
     def locus(self, locus_name):
-        """Set the locus name is set as the name of the Sequence.
-        """
+        """Set the locus name is set as the name of the Sequence."""
         self.data.name = locus_name
 
     def size(self, content):
@@ -671,18 +666,30 @@ class _FeatureConsumer(_BaseGenBankConsumer):
         """
         self._seq_type = type.strip()
 
-    def topology(self, topology):
-        """Record the topology (linear or circular as strings)."""
+    def topology(self, topology):  # noqa: D402
+        """Validate and record sequence topology (linear or circular as strings)."""
         if topology:
             if topology not in ['linear', 'circular']:
                 raise ParserFailureError("Unexpected topology %r should be linear or circular" % topology)
             self.data.annotations['topology'] = topology
 
     def molecule_type(self, mol_type):
-        """Record the molecule type (for round-trip etc)."""
+        """Validate and record the molecule type (for round-trip etc)."""
         if mol_type:
             if "circular" in mol_type or 'linear' in mol_type:
                 raise ParserFailureError("Molecule type %r should not include topology" % mol_type)
+
+            # Writing out records will fail if we have a lower case DNA
+            # or RNA string in here, so upper case it.
+            # This is a bit ugly, but we don't want to upper case e.g.
+            # the m in mRNA, but thanks to the strip we lost the spaces
+            # so we need to index from the back
+            if mol_type[-3:].upper() in ('DNA', 'RNA') and not mol_type[-3:].isupper():
+                import warnings
+                from Bio import BiopythonParserWarning
+                warnings.warn("Non-upper case molecule type in LOCUS line: %s"
+                              % mol_type, BiopythonParserWarning)
+
             self.data.annotations['molecule_type'] = mol_type
 
     def data_file_division(self, division):
@@ -692,8 +699,7 @@ class _FeatureConsumer(_BaseGenBankConsumer):
         self.data.annotations['date'] = submit_date
 
     def definition(self, definition):
-        """Set the definition as the description of the sequence.
-        """
+        """Set the definition as the description of the sequence."""
         if self.data.description:
             # Append to any existing description
             # e.g. EMBL files with two DE lines.
@@ -855,8 +861,7 @@ class _FeatureConsumer(_BaseGenBankConsumer):
         self.data.annotations['organism'] = content
 
     def taxonomy(self, content):
-        """Records (another line of) the taxonomy lineage.
-        """
+        """Record (another line of) the taxonomy lineage."""
         lineage = self._split_taxonomy(content)
         try:
             self.data.annotations['taxonomy'].extend(lineage)
@@ -864,8 +869,7 @@ class _FeatureConsumer(_BaseGenBankConsumer):
             self.data.annotations['taxonomy'] = lineage
 
     def reference_num(self, content):
-        """Signal the beginning of a new reference object.
-        """
+        """Signal the beginning of a new reference object."""
         # if we have a current reference that hasn't been added to
         # the list of references, add it.
         if self._cur_reference is not None:
@@ -916,7 +920,7 @@ class _FeatureConsumer(_BaseGenBankConsumer):
         self._cur_reference.location = all_locations
 
     def _split_reference_locations(self, location_string):
-        """Get reference locations out of a string of reference information
+        """Get reference locations out of a string of reference information (PRIVATE).
 
         The passed string should be of the form::
 
@@ -990,13 +994,11 @@ class _FeatureConsumer(_BaseGenBankConsumer):
         self.data.annotations['structured_comment'] = content
 
     def features_line(self, content):
-        """Get ready for the feature table when we reach the FEATURE line.
-        """
+        """Get ready for the feature table when we reach the FEATURE line."""
         self.start_feature_table()
 
     def start_feature_table(self):
-        """Indicate we've got to the start of the feature table.
-        """
+        """Indicate we've got to the start of the feature table."""
         # make sure we've added on our last reference object
         if self._cur_reference is not None:
             self.data.annotations['references'].append(self._cur_reference)
@@ -1045,9 +1047,35 @@ class _FeatureConsumer(_BaseGenBankConsumer):
         if _re_simple_location.match(location_line):
             # e.g. "123..456"
             s, e = location_line.split("..")
-            cur_feature.location = SeqFeature.FeatureLocation(int(s) - 1,
-                                                              int(e),
-                                                              strand)
+            try:
+                cur_feature.location = SeqFeature.FeatureLocation(int(s) - 1,
+                                                                  int(e),
+                                                                  strand)
+            except ValueError:
+                # Could be non-integers, more likely bad origin wrapping
+                import warnings
+                from Bio import BiopythonParserWarning
+                if int(s) > int(e) and "circular" in self._seq_type.lower():
+                    warnings.warn("Attempting to fix invalid location %r as "
+                                  "it looks like incorrect origin wrapping. "
+                                  "Please fix input file, this could have "
+                                  "unintended behavior." % location_line,
+                                  BiopythonParserWarning)
+
+                    f1 = SeqFeature.FeatureLocation(int(s) - 1,
+                                                    self._expected_size,
+                                                    strand)
+                    f2 = SeqFeature.FeatureLocation(0, int(e), strand)
+
+                    if strand == -1:
+                        # For complementary features spanning the origin
+                        cur_feature.location = f2 + f1
+                    else:
+                        cur_feature.location = f1 + f2
+                else:
+                    warnings.warn("Ignoring invalid location: %r" %
+                                  location_line, BiopythonParserWarning)
+                    cur_feature.location = None
             return
 
         if ",)" in location_line:
@@ -1166,7 +1194,19 @@ class _FeatureConsumer(_BaseGenBankConsumer):
             # otherwise just skip this key
             return
 
-        value = value.replace('"', '')
+        # Remove enclosing quotation marks
+        value = re.sub('^"|"$', '', value)
+
+        # Handle NCBI escaping
+        # Warn if escaping is not according to standard
+        if re.search(r'[^"]"[^"]|^"[^"]|[^"]"$', value):
+            import warnings
+            from Bio import BiopythonParserWarning
+            warnings.warn('The NCBI states double-quote characters like " should be escaped as "" '
+                          '(two double - quotes), but here it was not: %r' % value, BiopythonParserWarning)
+        # Undo escaping, repeated double quotes -> one double quote
+        value = value.replace('""', '"')
+
         if self._feature_cleaner is not None:
             value = self._feature_cleaner.clean_value(key, value)
 
@@ -1224,16 +1264,16 @@ class _FeatureConsumer(_BaseGenBankConsumer):
         self._seq_data.append(content.upper())
 
     def record_end(self, content):
-        """Clean up when we've finished the record.
-        """
+        """Clean up when we've finished the record."""
         from Bio import Alphabet
         from Bio.Alphabet import IUPAC
         from Bio.Seq import Seq, UnknownSeq
 
         # Try and append the version number to the accession for the full id
         if not self.data.id:
-            assert 'accessions' not in self.data.annotations, \
-                   self.data.annotations['accessions']
+            if 'accessions' in self.data.annotations:
+                raise ValueError("Problem adding version number to accession: "
+                                 + str(self.data.annotations['accessions']))
             self.data.id = self.data.name  # Good fall back?
         elif self.data.id.count('.') == 0:
             try:
@@ -1290,8 +1330,8 @@ class _FeatureConsumer(_BaseGenBankConsumer):
 
 
 class _RecordConsumer(_BaseGenBankConsumer):
-    """Create a GenBank Record object from scanner generated information (PRIVATE).
-    """
+    """Create a GenBank Record object from scanner generated information (PRIVATE)."""
+
     def __init__(self):
         _BaseGenBankConsumer.__init__(self)
         from . import Record
@@ -1374,8 +1414,7 @@ class _RecordConsumer(_BaseGenBankConsumer):
         self.data.taxonomy = self._split_taxonomy(content)
 
     def reference_num(self, content):
-        """Grab the reference number and signal the start of a new reference.
-        """
+        """Grab the reference number and signal the start of a new reference."""
         # check if we have a reference to add
         if self._cur_reference is not None:
             self.data.references.append(self._cur_reference)
@@ -1421,27 +1460,24 @@ class _RecordConsumer(_BaseGenBankConsumer):
         self.data.structured_comment = content
 
     def primary_ref_line(self, content):
-        """Data for the PRIMARY line"""
+        """Save reference data for the PRIMARY line."""
         self.data.primary.append(content)
 
     def primary(self, content):
         pass
 
     def features_line(self, content):
-        """Get ready for the feature table when we reach the FEATURE line.
-        """
+        """Get ready for the feature table when we reach the FEATURE line."""
         self.start_feature_table()
 
     def start_feature_table(self):
-        """Signal the start of the feature table.
-        """
+        """Signal the start of the feature table."""
         # we need to add on the last reference
         if self._cur_reference is not None:
             self.data.references.append(self._cur_reference)
 
     def feature_key(self, content):
-        """Grab the key of the feature and signal the start of a new feature.
-        """
+        """Grab the key of the feature and signal the start of a new feature."""
         # first add on feature information if we've got any
         self._add_feature()
 
@@ -1450,7 +1486,7 @@ class _RecordConsumer(_BaseGenBankConsumer):
         self._cur_feature.key = content
 
     def _add_feature(self):
-        """Utility function to add a feature to the Record.
+        """Add a feature to the record, with relevant checks (PRIVATE).
 
         This does all of the appropriate checking to make sure we haven't
         left any info behind, and that we are only adding info if it
@@ -1474,7 +1510,7 @@ class _RecordConsumer(_BaseGenBankConsumer):
             self.feature_qualifier_description(value)
 
     def feature_qualifier_name(self, content_list):
-        """Deal with qualifier names
+        """Deal with qualifier names.
 
         We receive a list of keys, since you can have valueless keys such as
         /pseudo which would be passed in with the next key (since no other
@@ -1511,8 +1547,7 @@ class _RecordConsumer(_BaseGenBankConsumer):
         self.data.origin = content
 
     def contig_location(self, content):
-        """Signal that we have contig information to add to the record.
-        """
+        """Signal that we have contig information to add to the record."""
         self.data.contig = self._clean_location(content)
 
     def sequence(self, content):
@@ -1527,8 +1562,7 @@ class _RecordConsumer(_BaseGenBankConsumer):
         self._seq_data.append(content.upper())
 
     def record_end(self, content):
-        """Signal the end of the record and do any necessary clean-up.
-        """
+        """Signal the end of the record and do any necessary clean-up."""
         # add together all of the sequence parts to create the
         # final sequence string
         self.data.sequence = "".join(self._seq_data)
