@@ -1,8 +1,8 @@
 # Copyright 2012 by Wibowo Arindrarto.  All rights reserved.
-# This code is part of the Biopython distribution and governed by its
-# license.  Please see the LICENSE file that should have been included
-# as part of this package.
-
+# This file is part of the Biopython distribution and governed by your
+# choice of the "Biopython License Agreement" or the "BSD 3-Clause License".
+# Please see the LICENSE file that should have been included as part of this
+# package.
 """Bio.SearchIO abstract base parser for Exonerate standard output format."""
 
 import re
@@ -22,13 +22,13 @@ _RE_TRANS = re.compile(r'[53ISCF]')
 
 
 def _set_frame(frag):
-    """Sets the HSPFragment frames."""
+    """Set the HSPFragment frames (PRIVATE)."""
     frag.hit_frame = (frag.hit_start % 3 + 1) * frag.hit_strand
     frag.query_frame = (frag.query_start % 3 + 1) * frag.query_strand
 
 
 def _make_triplets(seq, phase=0):
-    """Selects a valid amino acid sequence given a 3-letter code input.
+    """Select a valid amino acid sequence given a 3-letter code input (PRIVATE).
 
     This function takes a single three-letter amino acid sequence and the phase
     of the sequence to return the longest intact amino acid sequence possible.
@@ -57,7 +57,7 @@ def _make_triplets(seq, phase=0):
 
 
 def _get_fragments_coord(frags):
-    """Returns the letter coordinate of the given list of fragments.
+    """Return the letter coordinate of the given list of fragments (PRIVATE).
 
     This function takes a list of three-letter amino acid sequences and
     returns a list of coordinates for each fragment had all the input
@@ -84,7 +84,7 @@ def _get_fragments_coord(frags):
 
 
 def _get_fragments_phase(frags):
-    """Returns the phases of the given list of 3-letter amino acid fragments.
+    """Return the phases of the given list of 3-letter amino acid fragments (PRIVATE).
 
     This is an internal private function and is meant for parsing Exonerate's
     three-letter amino acid output.
@@ -106,8 +106,9 @@ def _get_fragments_phase(frags):
 
 
 def _adjust_aa_seq(fraglist):
-    """Transforms three-letter amino acid codes into one-letters in the
-    given HSPFragments.
+    """Transform 3-letter AA codes of input fragments to one-letter codes (PRIVATE).
+
+    Argument fraglist should be a list of HSPFragments objects.
     """
     custom_map = {'***': '*', '<->': '-'}
     hsp_hstart = fraglist[0].hit_start
@@ -171,7 +172,7 @@ def _adjust_aa_seq(fraglist):
 
 
 def _split_fragment(frag):
-    """Splits one HSPFragment containing frame-shifted alignment into two."""
+    """Split one HSPFragment containing frame-shifted alignment into two (PRIVATE)."""
     # given an HSPFragment object with frameshift(s), this method splits it
     # into fragments without frameshifts by sequentially chopping it off
     # starting from the beginning
@@ -202,9 +203,9 @@ def _split_fragment(frag):
         # coordinates for the split strand
         qstart, hstart = qpos, hpos
         qpos += (len(split) - sum(str(split.query.seq).count(x)
-            for x in ('-', '<', '>'))) * qstep
+                 for x in ('-', '<', '>'))) * qstep
         hpos += (len(split) - sum(str(split.hit.seq).count(x)
-            for x in ('-', '<', '>'))) * hstep
+                 for x in ('-', '<', '>'))) * hstep
 
         split.hit_start = min(hstart, hpos)
         split.query_start = min(qstart, qpos)
@@ -235,7 +236,7 @@ def _split_fragment(frag):
 
 
 def _create_hsp(hid, qid, hspd):
-    """Returns a list of HSP objects from the given parsed HSP values."""
+    """Return a list of HSP objects from the given parsed HSP values (PRIVATE)."""
     frags = []
     # we are iterating over query_ranges, but hit_ranges works just as well
     for idx, qcoords in enumerate(hspd['query_ranges']):
@@ -282,7 +283,7 @@ def _create_hsp(hid, qid, hspd):
     hsp = HSP(frags)
     # set hsp-specific attributes
     for attr in ('score', 'hit_split_codons', 'query_split_codons',
-            'model', 'vulgar_comp', 'cigar_comp', 'alphabet'):
+                 'model', 'vulgar_comp', 'cigar_comp', 'alphabet'):
         if attr in hspd:
             setattr(hsp, attr, hspd[attr])
 
@@ -290,7 +291,7 @@ def _create_hsp(hid, qid, hspd):
 
 
 def _parse_hit_or_query_line(line):
-    """Parse the 'Query:' line of exonerate alignment outputs."""
+    """Parse the 'Query:' line of exonerate alignment outputs (PRIVATE)."""
     try:
         mark, id, desc = line.split(' ', 2)
     except ValueError:  # no desc
@@ -322,7 +323,7 @@ class _BaseExonerateParser(object):
                     self.line.startswith('cigar:'):
                 break
             elif not self.line or self.line.startswith('-- completed '):
-                raise StopIteration
+                return
 
         for qresult in self._parse_qresult():
             qresult.program = 'exonerate'
@@ -333,7 +334,7 @@ class _BaseExonerateParser(object):
             yield qresult
 
     def read_until(self, bool_func):
-        """Reads the file handle until the given bool function returns True."""
+        """Read the file handle until the given bool function returns True."""
         while True:
             if not self.line or bool_func(self.line):
                 return
@@ -431,7 +432,7 @@ class _BaseExonerateParser(object):
                 # if the file has c4 alignments, try to parse the header
                 if self.has_c4_alignment:
                     self.read_until(lambda line:
-                            line.strip().startswith('Query:'))
+                                    line.strip().startswith('Query:'))
                     header = self._parse_alignment_header()
                 # parse the block contents
                 cur = self.parse_alignment_block(header)
@@ -493,7 +494,7 @@ class _BaseExonerateIndexer(SearchIndexer):
         raise NotImplementedError("Should be defined by subclass")
 
     def __iter__(self):
-        """Iterates over the file handle; yields key, start offset, and length."""
+        """Iterate over the file handle; yields key, start offset, and length."""
         handle = self._handle
         handle.seek(0)
         qresult_key = None
